#!/usr/bin/env perl
#use lib "/home/01207/em23799/bin/BioPerl-1.6.0";
use Bio::Perl;
use Bio::SeqIO;
use Bio::SearchIO;
#use Bio::DB::Fasta;

# -- program name
print "\n","-"x60, "\n";
print "454trim_blast.pl v 1.0\n";
print "Created 12 Oct 2011 M Matz\n";
print "Last modified 12 Oct 2011\n";
print "-"x60, "\n";

# -- program description and required arguments
unless ($#ARGV >= 1)
        {print "Identifies adaptor-matching ends to trim in 454 reads\n";
        print "Output:\t tab delimited text file with read ID, start position, end position\n";
        print "Usage:\t script adaptors reads\n";
        print "Arguments:\n";
        print "\t 0. adaptors\t fasta file of adaptor sequences \n";
        print "\t 1. reads\t\t fasta file of raw reads (extracted from SFF file using sffinfo)\n";
        print "\t 2. min. length\t\t minimal length of the trimmed read; default 50 \n";
        print "\t 3. evalue\t\t for blastn; default 0.1 \n";
        print "\t 4. word size\t\t for blastn; default 7\n";
        print "\t 5. alpha\t\t the start nucleotide of the read; default 1 (increase it if leftover adaptor/barcode present) \n";
        print "\n"; exit;
        }
my $minlen=50;
my $eval=0.1;
my $word=7;
my $alpha0=1;
if ($ARGV[2]) { $minlen = $ARGV[2] ;}
if ($ARGV[3]) { $eval = $ARGV[3] ;}
if ($ARGV[4]) { $word = $ARGV[4]; }
if ($ARGV[5]) { $alpha0 = $ARGV[5]; }
my $hitno = 30;
my $maxread=500;
my $counts=0;
$outname="trimmed_".$ARGV[1].".tab";
open OUTTAB,">$outname";

#$problems=" GXGRU2304J0082 GXGRU2304JZC2X GXGRU2304I5GIR ";

# doing it one seq at a time

#`formatdb -i $ARGV[0] -o T -p F`;
my $rseqs = new Bio::SeqIO(-file=>$ARGV[1], -format=>"fasta");
while($rseq = $rseqs->next_seq)	{
 	$rlen = $rseq->length;
	if ( $rlen < $minlen) { next;}
 	$read= $rseq->display_id;
#print "-"x60,"\n$read\n";
    my $oneread = Bio::SeqIO->new(-file => '>1read.fna', -format => 'fasta' );
    $oneread->write_seq($rseq);
      my $brnam=$read.".br";

	`blastall -d $ARGV[0] -p blastn  -a 8 -W $word -F F -e $eval -v $hitno  -b $hitno -i 1read.fna -o $brnam`;

	my $br = new Bio::SearchIO(-format => 'blast',  -file   => $brnam);
	my @hsps=();
	my @rehsp=();
	my $result=$br->next_result;

# collecting hsps of matches to adaptors

	while( my $hit = $result->next_hit ) {
		while( my $hsp = $hit->next_hsp ) {
			if ($hsp->evalue > $eval) { next;}
			push @hsps, $hsp->start('query') . "_" . $hsp->end('query');
		}
	}

# grouping hsps into overlapping groups with less than "minlen" of uncovered sequence in between (or towards the end)

	@hsps=sort { $a <=> $b} @hsps;
	my @merge=();
	for ($h=0; $hsp1=$hsps[$h];$h++) {
		if ($merge[$h]) {next;}
		($s1,$e1)=split(/_/,$hsp1);
		if ($s1<$minlen) { $s1=1;}
		if ( $rlen - $e1 < $minlen ) { $e1=$rlen;}
		for ($h2=$h+1; $hsp2=$hsps[$h2];$h2++) {
			if ($merge[$h2]) {next;}
			($s2,$e2)=split(/_/,$hsp2);
			if ($s2<$minlen) { $s2=1;}
			if ( $rlen - $e2 < $minlen ) { $e2=$rlen;}
			if ($s2-$e1<$minlen)	{ 
#print "\t\tmerging: $h: ",$s1,"_",$e1," / $h2: $hsp2  => ";
				$e1=$e2 unless ($e1>$e2);
#print $s1,"_",$e1,"\n";
				$merge[$h2]="Y";
			}
		}
		push @rehsp, $s1."_".$e1;
	}
print "\n$read\nlength:\t$rlen\nadaptors:\t@rehsp\nad.segments:\t",$#rehsp+1,"\n";

# writing output table 
	
	my $alpha=$alpha0;
	my $omega=$rlen;
	if ($rehsp[0]){
		($ii1,$end0)=split /_/, $rehsp[0];
		if ($rehsp[1]){
			if ($ii1<$minlen || ($ii1-$alpha)<$minlen) {
				if ($alpha<$end0){$alpha=$end0+1;}
				($omega,$ii2)=split /_/, $rehsp[1];
				$omega--;
			}
			else { $omega=$ii1-1;}
		}
		elsif ($ii1<$minlen || ($ii1-$alpha)<$minlen) {
			if ($alpha<$end0){$alpha=$end0+1;}
		}
		else { $omega=$ii1-1; }
	}
	if ($omega-$alpha>$minlen){
print "selecting\t",$alpha,"_",$omega,"\n";
		print OUTTAB "$read\t$alpha\t$omega\n";
	}
	`rm $brnam` ; #unless ($problems=~/\s$read\s/ || $counts<20 );
	$counts++;
}
close OUTTAB;


