#!/usr/bin/perl

$usage="

mBC_count:

counts reads mapping to OTUs in SAM files

Arguments:

sam=[filename]        : SAM file
mult.hits=toss|random : if a read maps to multiple reference sequences, it is disregarded 
                        by default. Set this option to 'random' if you want to randomly 
                        pick an OTU to assign the count to. In my opinion this makes sense 
                        only for OTUs that are approximately equally represented.

";

my $t1="";
if ("@ARGV"=~/input=(\S+)/) { $t1=$1;}
else { die $usage; }
my $miso="toss";
if ("@ARGV"=~/mult.hits=random/) { 
	$miso="random";
	warn "adding a count to a randomly picked OTU when a read maps to multiple OTUs\n";
}
else { warn "disregarding reads mapping to multiple OTUs\n"; }

open SAM, $t1 or die "cannot open SAM file $t1\n";

my %count={};
my %hit={};
my %refhit={};
my $c="";
my $f="";
my $r="";
my $pos="";
my $seq="";
my $flag="";
my $mapq="";
my @rest=();

while (<SAM>) {
	if ($_=~/^@/) { next;}
	chop;
	($r,$flag,$c,$pos,$mapq,@rest)=split(/\s/,$_);
	if ($r=~/@/) {next;}
	my @sseq=grep(/[ATGCatgc-]{30,}/,@rest);
	next if (!$sseq[0]);
	$seq=$sseq[0];
	push @{$refhit{$c}{$pos}},$seq;
	push @{$hit{$r}},$c ; 
}

foreach $r (keys %hit){
	next if ($r=~/HASH/);
	if($#{$hit{$r}}>0) {
#warn "--------\n$r many hits:\n@{$hit{$r}}\n";
		if($miso eq "random") {
			my $pick=${$hit{$r}}[rand @{$hit{$r}}];
			$count{$pick}++;
#warn "picked $pick\n";
		}
	}
	else { 
#warn "--------\n$r one hit:\n\t@{$hit{$r}}\n\tadding ${$hit{$r}}[0]\n";
		$count{${$hit{$r}}[0]}++;
	}
}

foreach $c (sort keys %count) {
	next if ($c=~/HASH/);
	print "$c\t$count{$c}\n";
}
